@@ -52,7 +52,7 @@ class DataSet(pd.DataFrame):
             self._spacegroup = spacegroup
         if cell:
             self._cell = cell
-        if merged:
+        if merged is not None:
             self._merged = merged
             
         # Build DataSet using DataFrame.__init__()
@@ -101,47 +101,125 @@ class DataSet(pd.DataFrame):
     #-------------------------------------------------------------------
     # Methods
     
-    def set_index(self, keys, **kwargs):
+    def set_index(self, keys, drop=True, append=False, inplace=False, verify_integrity=False):
+        """
+        Set the DataSet index using existing columns.
+
+        Set the DataSet index (row labels) using one or more existing 
+        columns or arrays (of the correct length). The index can replace
+        the existing index or expand on it.
+
+        Parameters
+        ----------
+        keys : label or array-like or list of labels/arrays
+            This parameter can be either a single column key, a single 
+            array of the same length as the calling DataSet, or a list 
+            containing an arbitrary combination of column keys and arrays. 
+        drop : bool
+            Whether to delete columns to be used as the new index.
+        append : bool
+            Whether to append columns to existing index.
+        inplace  : bool
+            Modify the DataFrame in place (do not create a new object).
+        verify_integrity : bool
+            Check the new index for duplicates. Otherwise defer the check
+            until necessary. Setting to False will improve the performance 
+            of this method
+
+        Returns
+        -------
+        DataSet or None
+            DataSet with the new index or None if `inplace=True`
+
+        See Also
+        --------
+        DataSet.reset_index : Reset index
+        """
+        if not isinstance(keys, list):
+            keys = [keys]
         
         # Copy dtypes of keys to cache
         for key in keys:
-            self._cache_index_dtypes[key] = self[key].dtype.name
+            if isinstance(key, str):
+                self._cache_index_dtypes[key] = self[key].dtype.name
+            elif isinstance(key, (np.ndarray, pd.Index, pd.Series)):
+                self._cache_index_dtypes = key.dtype.name
+            elif isinstance(key, list):
+                self._cache_index_dtypes = type(key[0])
+            else:
+                raise ValueError(f"{key} is not an instance of type str, np.ndarray, pd.Index, pd.Series, or list")
+            
+        return super().set_index(keys, drop, append, inplace, verify_integrity)
 
-        return super().set_index(keys, **kwargs)
+    def reset_index(self, level=None, drop=False, inplace=False, col_level=0, col_fill=''):
+        """
+        Reset the index or a specific level of a MultiIndex.
 
-    def reset_index(self, **kwargs):
+        Reset the index to use a numbered RangeIndex. Using the `level`
+        argument, it is possible to reset one or more levels of a MultiIndex.
+
+        Parameters
+        ----------
+        level : int, str, tuple, list
+            Only remove given levels from the index. Defaults to all levels
+        drop : bool
+            Do not try to insert index into dataframe columns.
+        inplace ; bool
+            Modify the DataSet in place (do not create a new object).
+        col_level : int or str
+            If the columns have multiple levels, determines which level 
+            the labels are inserted into. By default it is inserted into
+            the first level.
+        col_fill : object
+            If the columns have multiple levels, determines how the other
+            levels are named. If None then the index name is repeated.
+
+        Returns
+        -------
+        DataSet or None
+            DataSet with the new index or None if `inplace=True`
+
+        See Also
+        --------
+        DataSet.set_index : Set index
+        """
         
-        if kwargs.get("inplace", False):
-            super().reset_index(**kwargs)
-
-            # Cast all values to cached dtypes
-            if not kwargs.get("drop", False):
-                for key in self._cache_index_dtypes.keys():
-                    dtype = self._cache_index_dtypes[key]
-                    self[key] = self[key].astype(dtype)
-                self._cache_index_dtypes = {}
+        # GH#6: Handle level argument to reset_index
+        columns = level
+        if level is None:
+            columns = list(self._cache_index_dtypes.keys())
+        
+        def _handle_cached_dtypes(dataset, columns, drop):
+            """Use _cache_index_dtypes to restore dtypes"""
+            if drop:
+                for key in columns:
+                    dataset._cache_index_dtypes.pop(key)
+            else:
+                for key in columns:
+                    dtype = dataset._cache_index_dtypes.pop(key)
+                    dataset[key] = dataset[key].astype(dtype)
+            return dataset
+        
+        if inplace:
+            super().reset_index(level, drop, inplace, col_level, col_fill)
+            _handle_cached_dtypes(self, columns, drop)
             return
-        
         else:
-            newdf = super().reset_index(**kwargs)
-
-            # Cast all values to cached dtypes
-            if not kwargs.get("drop", False):
-                for key in newdf._cache_index_dtypes.keys():
-                    dtype = newdf._cache_index_dtypes[key]
-                    newdf[key] = newdf[key].astype(dtype)
-                newdf._cache_index_dtypes = {}
-            return newdf
+            dataset = super().reset_index(level, drop, inplace, col_level, col_fill)
+            dataset._cache_index_dtypes = dataset._cache_index_dtypes.copy()
+            dataset = _handle_cached_dtypes(dataset, columns, drop)
+            return dataset
 
     @classmethod
     def from_gemmi(cls, gemmiMtz):
         """
         Creates DataSet object from gemmi.Mtz object.
 
-        If the gemmi.Mtz object contains an M/ISYM column, an unmerged
-        DataSet will be constructed. The Miller indices will be mapped to
-        their observed values, and a partiality flag will be extracted 
-        and stored as a boolean column with the label, ``PARTIAL``. 
+        If the gemmi.Mtz object contains an M/ISYM column and contains duplicated
+        Miller indices, an unmerged DataSet will be constructed. The Miller indices 
+        will be mapped to their observed values, and a partiality flag will be 
+        extracted and stored as a boolean column with the label, ``PARTIAL``. 
+        Otherwise, a merged DataSet will be constructed.
 
         If columns are found with the ``MTZInt`` dtype and are labeled ``PARTIAL``
         or ``CENTRIC``, these will be interpreted as boolean flags used to 
@@ -181,6 +259,116 @@ class DataSet(pd.DataFrame):
         from reciprocalspaceship import io
         return io.to_gemmi(self, skip_problem_mtztypes)
     
+    def append(self, *args, check_isomorphous=True, **kwargs):
+        """
+        Append rows of `other` to the end of calling DataSet, returning
+        a new DataSet object. Any columns in `other` that are not present
+        in the calling DataSet are added as new columns. 
+
+        For additional documentation on accepted arguments, see the 
+        `Pandas DataFrame.append() API <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.append.html>`_.
+
+        Parameters
+        ----------
+        check_isomorphous : bool
+            If True, the spacegroup and cell attributes of DataSets in `other`
+            will be compared to those of the calling DataSet to ensure
+            they are isomorphous. 
+
+        Returns
+        -------
+        rs.DataSet
+
+        See Also
+        --------
+        concat : Concatenate ``rs`` objects
+        """
+        other = kwargs.get("other", args[0])
+        if check_isomorphous:
+            if isinstance(other, (list, tuple)):
+                for o in other:
+                    if isinstance(o, DataSet) and not self.is_isomorphous(o):
+                        raise ValueError("DataSet in `other` is not isomorphous")
+            else:
+                if isinstance(other, DataSet) and not self.is_isomorphous(other):
+                    raise ValueError("`other` DataSet is not isomorphous")
+        result = super().append(*args, **kwargs)
+
+        # If `ignore_index=True`, the _cache_index_dtypes attribute should
+        # be reset.
+        if isinstance(result.index, pd.RangeIndex) and self._cache_index_dtypes != {}:
+            result.__finalize__(self)
+            result._cache_index_dtypes = {}
+            return result
+        
+        return result.__finalize__(self)
+
+    def merge(self, *args, check_isomorphous=True, **kwargs):
+        """
+        Merge DataSet or named DataSeries using a database-style join on
+        columns or indices.
+
+        For additional documentation on accepted arguments, see the 
+        `Pandas DataFrame.merge() API <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.merge.html>`_.
+
+        Parameters
+        ----------
+        check_isomorphous : bool
+            If True, the spacegroup and cell attributes of DataSets in `other`
+            will be compared to those of the calling DataSet to ensure
+            they are isomorphous. 
+
+        Returns
+        -------
+        rs.DataSet
+
+        See Also
+        --------
+        DataSet.join : Similar method with support for lists of ``rs`` objects
+        """
+        right = kwargs.get("right", args[0])
+        if check_isomorphous and isinstance(right, DataSet):
+            if not self.is_isomorphous(right):
+                raise ValueError("`right` DataSet is not isomorphous")
+        result = super().merge(*args, **kwargs)
+        return result.__finalize__(self)
+
+    def join(self, *args, check_isomorphous=True, **kwargs):
+        """
+        Join DataSets or named DataSeries using a database-style join on
+        columns or indices. This method can be used to join lists ``rs`` objects
+        to a given DataSet. 
+
+        For additional documentation on accepted arguments, see the 
+        `Pandas DataFrame.join() API <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.join.html>`_.
+
+        Parameters
+        ----------
+        check_isomorphous : bool
+            If True, the spacegroup and cell attributes of DataSets in `other`
+            will be compared to those of the calling DataSet to ensure
+            they are isomorphous. 
+
+        Returns
+        -------
+        rs.DataSet
+
+        See Also
+        --------
+        DataSet.merge : Similar method with added flexibility for distinct column labels
+        """
+        other = kwargs.get("other", args[0])
+        if check_isomorphous:
+            if isinstance(other, (list, tuple)):
+                for o in other:
+                    if isinstance(o, DataSet) and not self.is_isomorphous(o):
+                        raise ValueError("DataSet in `other` is not isomorphous")
+            else:
+                if isinstance(other, DataSet) and not self.is_isomorphous(other):
+                    raise ValueError("`other` DataSet is not isomorphous")
+        result = super().join(*args, **kwargs)
+        return result.__finalize__(self)
+    
     def write_mtz(self, mtzfile, skip_problem_mtztypes=False):
         """
         Write DataSet to MTZ file.
@@ -233,12 +421,14 @@ class DataSet(pd.DataFrame):
 
         Parameters
         ----------
-        symop : gemmi.Op
-            Gemmi symmetry operation
+        symop : str, gemmi.Op
+            Gemmi symmetry operation or string representing symmetry op
         inplace : bool
             Whether to return a new DataFrame or make the change in place
         """
-        if not isinstance(symop, gemmi.Op):
+        if isinstance(symop, str):
+            symop = gemmi.Op(symop)
+        elif not isinstance(symop, gemmi.Op):
             raise ValueError(f"Provided symop is not of type gemmi.Op")
 
         if inplace:
@@ -356,6 +546,38 @@ class DataSet(pd.DataFrame):
         dataset['dHKL'] = rs.DataSeries(dHKL, dtype='R', index=dataset.index)
         return dataset
 
+    def assign_resolution_bins(self, bins=20, inplace=False, return_labels=True):
+        """
+        Assign reflections in DataSet to resolution bins.
+
+        Parameters
+        ----------
+        bins : int
+            Number of bins
+        inplace : bool
+            Whether to add the column in place or return a copy
+        return_labels : bool
+            Whether to return a list of labels corresponding to the edges
+            of each resolution bin
+
+        Returns
+        -------
+        (DataSet, list) or DataSet
+        """
+        if inplace:
+            dataset = self
+        else:
+            dataset = self.copy()
+        dHKL = dataset.compute_dHKL()["dHKL"]
+
+        assignments, labels = rs.utils.bin_by_percentile(dHKL, bins=bins, ascending=False)
+        dataset["bin"] = rs.DataSeries(assignments, dtype="I", index=dataset.index)
+
+        if return_labels:
+            return dataset, labels
+        else:
+            return dataset
+        
     def stack_anomalous(self, plus_labels=None, minus_labels=None):
         """
         Convert data from two-column anomalous format to one-column
@@ -535,6 +757,48 @@ class DataSet(pd.DataFrame):
             
         return result.__finalize__(self)
         
+    def is_isomorphous(self, other, cell_threshold=0.05):
+        """
+        Determine whether DataSet is isomorphous to another DataSet. This
+        method confirms isomorphism by ensuring the spacegroups are equivalent,
+        and that the cell parameters are within a specified percentage 
+        (see `cell_threshold`). 
+
+        Parameters
+        ----------
+        other : rs.DataSet
+            DataSet to which it will be compared
+        cell_threshold : float
+            Acceptable percent difference between unit cell parameters
+
+        Returns
+        -------
+        bool
+        """
+        # Check for spacegroup and cell attributes
+        if not isinstance(self.spacegroup, gemmi.SpaceGroup):
+            raise AttributeError(f"Calling dataset's spacegroup attribute is not set")
+        elif not isinstance(other.spacegroup, gemmi.SpaceGroup):
+            raise AttributeError(f"`other` dataset's spacegroup attribute is not set")
+        elif not isinstance(self.cell, gemmi.UnitCell):
+            raise AttributeError(f"Calling dataset's cell attribute is not set")
+        elif not isinstance(other.cell, gemmi.UnitCell):
+            raise AttributeError(f"`other` dataset's cell attribute is not set")            
+        
+        # Check spacegroup
+        if self.spacegroup.xhm() != other.spacegroup.xhm():
+            return False
+
+        # Check cell parameters
+        params = ["a", "b", "c", "alpha", "beta", "gamma"]
+        for param in params:
+            param1 = self.cell.__getattribute__(param)
+            param2 = other.cell.__getattribute__(param)
+            if (np.abs((param1 - param2)) / 100.) > cell_threshold:
+                return False
+        
+        return True
+
     def hkl_to_asu(self, inplace=False):
         """
         Map HKL indices to the reciprocal space asymmetric unit. If phases
@@ -564,6 +828,11 @@ class DataSet(pd.DataFrame):
         else:
             dataset = self.copy()
 
+        index_keys = None
+        if "H" in dataset.index.names or "K" in dataset.index.names or "L" in dataset.index.names:
+            index_keys = dataset.index.names
+            dataset.reset_index(inplace=True)
+
         # Compute new HKLs and phase shifts
         hkls = dataset.get_hkls()
         compressed_hkls, inverse = np.unique(hkls, axis=0, return_inverse=True)
@@ -572,17 +841,19 @@ class DataSet(pd.DataFrame):
             dataset.spacegroup, 
             return_phase_shifts=True
         )
-        index_keys = dataset.index.names
-        dataset.reset_index(inplace=True)
+
+
         dataset[["H", "K", "L"]] = asu_hkls[inverse]
         dataset[["H", "K", "L"]] = dataset[["H", "K", "L"]].astype("HKL")
-        dataset.set_index(index_keys, inplace=True)
+
+        if index_keys is not None:
+            dataset.set_index(index_keys, inplace=True)
 
         # Apply phase shift
         for k in dataset.get_phase_keys():
             dataset[k] = phi_coeff[inverse] * (dataset[k] + phi_shift[inverse])
         dataset.canonicalize_phases(inplace=True)
-        
+
         # GH#3: if PARTIAL column exists, use it to construct M/ISYM
         if "PARTIAL" in dataset.columns:
             m_isym = isym[inverse] + 256*dataset["PARTIAL"].to_numpy()
