@@ -25,7 +25,7 @@ from .m_isym import M_IsymDtype
 # Since we want to overwrite a few of the one-letter strings, we need
 # to make sure that rs ExtensionDtypes appear first in the registry.
 # This will be handled by reversing the list.
-from pandas.core.dtypes.dtypes import registry
+from pandas.core.dtypes.base import registry
 registry.dtypes = registry.dtypes[::-1]
 
 from .summarize import summarize_mtz_dtypes
